<?php
namespace Controller;
use Exception;
use Traits\TQuery;
use Traits\TResponse;
use Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource;
use Util\InstanceProvider;
use Service\CustomScreen as CustomScreenService;

class CustomScreen extends Simple {

    use TResponse;

    /**
     * @var CustomScreenService
     */
    private $customScreenService;

    /**
     * CustomScreen constructor.
     * @param CustomScreenService $customScreenService
     */
    public function __construct(CustomScreenService $customScreenService)
    {
        $this->customScreenService = $customScreenService;
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function findProductCustom(DTO\Request $request, DTO\Response $response){
        try{
            $data = $this->customScreenService->findProductCustom();
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findElementProduct(DTO\Request $request, DTO\Response $response){
        try{
            $param = $request->getParameters();
            $productId = !empty($param['productId'])  ? $param['productId'] : null;
            $elementProductOrg = !empty($param['elementProductOrg'])  ? $param['elementProductOrg'] : null;
            $data = $this->customScreenService->findElementProduct($productId, $elementProductOrg);
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveElementCustom(DTO\Request $request, DTO\Response $response){
        try{
            $param = $request->getParameters();
            $nrprodutoId =  $param['NRPRODUTO_ID'];
            $nrprodutoOrgId = $param['NRPRODUTO_ORG_ID'];
            $label = $param['LABEL'];
            $parentUniqueId = $param['PARENT_UNIQUE_ID'];
            $parentId = $param['PARENT_ID'];
            $nrordem = $param['NRORDEM'];
            $route = $param['ROUTE'];
            $isNew = $param['ISNEW'];
            $this->customScreenService->saveElementCustom($nrprodutoId, $nrprodutoOrgId, $label, $parentUniqueId, $parentId, $nrordem, $route, $isNew);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function editElementCustom(DTO\Request $request, DTO\Response $response){
        try{
            $param = $request->getParameters();
            $id = $param['ID'];
            $label = $param['LABEL'];
            $nrordem = $param['NRORDEM'];
            $route = $param['ROUTE'];
            $parentId = $param['PARENT_ID'];
            $isNew = $param['ISNEW'];
            $nrprodutoId =  $param['NRPRODUTO_ID'];
            $nrprodutoOrgId = $param['NRPRODUTO_ORG_ID'];
            $this->customScreenService->editElementCustom($id, $label, $nrordem, $route, $parentId, $isNew, $nrprodutoId, $nrprodutoOrgId);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function removeElementCustom(DTO\Request $request, DTO\Response $response){
        try{
            $param = $request->getParameters();
            $id = $param['ID'];
            $this->customScreenService->removeElementCustom($id);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}