<?php
namespace Controller;
use Exception;
use Traits\TQuery;
use Traits\TResponse;
use Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource;
use Util\InstanceProvider;
use Service\OqueHaDeNovo as OqueHaDeNovoService;

class OqueHaDeNovo extends Simple {

    use TResponse;

    /**
     * @var OqueHaDeNovoService
     */
    private $oqueHaDeNovoService;

    private $fileServerFilesUrl;

    /**
     * OqueHaDeNovo constructor.
     * @param OqueHaDeNovoService $oqueHaDeNovoService
     */
    public function __construct(OqueHaDeNovoService $oqueHaDeNovoService, $fileServerFilesUrl)
    {
        $this->oqueHaDeNovoService = $oqueHaDeNovoService;
        $this->fileServerFilesUrl = $fileServerFilesUrl;
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function findTreeGridSubject(DTO\Request $request, DTO\Response $response){
        try{
            $data = $this->oqueHaDeNovoService->findTreeGridSubject($request->getParameters());
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function findItemProduct(DTO\Request $request, DTO\Response $response){
        try{
            $param = $request->getParameters();
            $assuntoId = !empty($param['ASSUNTO_ID'])  ? $param['ASSUNTO_ID'] : null;
            $dynamicFilter = !empty($param['dynamic_filter'])  ? $param['dynamic_filter'] : null;
            $data = $this->oqueHaDeNovoService->findItemProduct($dynamicFilter, $assuntoId);
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function downloadDocumentItem(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $request->getParameters();
            $id = $data['ID'];
            $dados = InstanceProvider::getRepository()->getOmOqaItem()->findOneBy(array('id' => $id));
            $url = $this->fileServerFilesUrl.$dados->getFilename();
            $this->sendResponse($url, 200);
        } catch (\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
