<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Factory\CustomScreen as CustomScreenFactory;
use Util\Utilities;

class CustomScreen {
    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var CustomScreenFactory
     */
    private $customScreenFactory;

    /** @var \Teknisa\Model\Teknisa\Repositories\OmProdutoOrgElementoCustom $omProdutoOrgElementoCustomRepository */
    protected  $omProdutoOrgElementoCustomRepository;

    /**
     * CustomScreen constructor.
     * @param EntityManager $entityManager
     * @param CustomScreenFactory $customScreenFactory
     */
    public function __construct(EntityManager $entityManager, CustomScreenFactory $customScreenFactory)
    {
        $this->entityManager = $entityManager;
        $this->customScreenFactory = $customScreenFactory;
        $this->omProdutoOrgElementoCustomRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\OmProdutoOrgElementoCustom');
    }

    public function findProductCustom()
    {
        $customerNrParc = InstanceProvider::getUtilities()->getCurrentCustomer()->getNrparcnegocio();
        return $this->customScreenFactory->findProductCustom($customerNrParc);
    }

    public function findElementProduct($productId, $elementProductOrg)
    {
        $data =  $this->customScreenFactory->findElementProduct($productId, $elementProductOrg);
        $data['data'] = Utilities::fixParentsForTreeGrid($data['data'], 'ID', 'PARENT_ID');
        return $data;
    }


    public function saveElementCustom($nrprodutoId, $nrprodutoOrgId, $label, $parentUniqueId, $parentId, $nrordem, $route, $isNew)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $validation = $this->validationFilhos($nrprodutoId, $nrprodutoOrgId, $parentId, $isNew);
            if ($validation === true) {
                throw new \Exception("ERROR");
            } else {
                $uniqueId = $this->getHashCode($nrprodutoId, $nrprodutoOrgId);
                $newElementCustom = $this->customScreenFactory->create($uniqueId, $nrprodutoOrgId, $label, $parentUniqueId, $parentId, $nrordem, $route);
                $this->entityManager->persist($newElementCustom);
                $this->entityManager->flush();
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function validationFilhos($productId, $elementProductOrg, $parentId, $isNew) {
        $verficaFilhosAux = $this->customScreenFactory->verficaFilhos($productId, $elementProductOrg, $parentId);
        $verficaFilhos = $verficaFilhosAux['data'];
        if (empty($verficaFilhos) && $isNew === 'N') {
            return true;
        }
        return false;
    }

    public function getHashCode($nrprodutoId, $nrprodutoOrgId) {
        $datas =  $this->customScreenFactory->findAllElementProduct($nrprodutoId, $nrprodutoOrgId);
        $datasAux = $datas['data'];
        foreach ($datasAux as $data) {
            $uniqueId = $this->getHashCodeRecursive($data['UNIQUE_ID']);
        }
        return $uniqueId;
    }

    public function getHashCodeRecursive($uniqueId) {
        $characters = '0123456789';
        $randomNumberAux = '';
        $randomNumber = '';
        for ($i = 0; $i < 23; $i++) {
            $randomNumberAux .= $characters[rand(0, strlen($characters) - 1)];
        }
        if ($uniqueId === $randomNumberAux) {
            $this->getHashCodeRecursive($uniqueId);
        } else {
            $randomNumber = $randomNumberAux;
        }
        return $randomNumber;
    }

    public function editElementCustom($id, $label, $nrordem, $route, $parentId, $isNew, $nrprodutoId, $nrprodutoOrgId)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $validation = $this->validationFilhos($nrprodutoId, $nrprodutoOrgId, $parentId, $isNew);
            if ($validation === true) {
                throw new \Exception("ERROR");
            } else {
                $produtoOrgElementoCustomRepository = $this->omProdutoOrgElementoCustomRepository->findOneBy(array('id' => $id));
                $editElementCustom = $this->customScreenFactory->update($produtoOrgElementoCustomRepository, $id, $label, $nrordem, $route, $parentId);
                $this->entityManager->persist($editElementCustom);
                $this->entityManager->flush();
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function removeElementCustom($id)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $this->removeFilhoElementoRecursivo($id);
            $produtoOrgElementoCustomRepository = $this->omProdutoOrgElementoCustomRepository->findOneBy(array('id' => $id));
            if (!empty($produtoOrgElementoCustomRepository)) {
                $this->entityManager->remove($produtoOrgElementoCustomRepository);
                $this->entityManager->flush();
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function removeFilhoElementoRecursivo($id) {
        $produtoOrgElementoCustomRepository = $this->omProdutoOrgElementoCustomRepository->findBy(array('parentId' => $id));
        foreach ($produtoOrgElementoCustomRepository as $filho) {
            $this->removeFilhoElementoRecursivo($filho->getId());
            $this->entityManager->remove($filho);
        }
    }
}